/*****************************************************************************
 *****************************************************************************
Copyright (c) 2000 - 2001, Intel Corporation 
  
All rights reserved.
Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
        
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
                  
 3. Neither the name of Intel Corporation nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.
                           
                         
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*****************************************************************************
****************************************************************************/

/**********************************************************************
 *                                                                     *
 * INTEL CORPORATION                                                   *
 *                                                                     *
 * This software is supplied under the terms of the license included   *
 * above.  All use of this software must be in accordance with the     *
 * terms of that license.                                              *
 *                                                                     *
 * Module Name:  diag.c		                                       *
 *                                                                     *
 * Abstract:     Header file for data structure definition for common  *
 *               diagnostics support                                   *
 *                                                                     *
 **********************************************************************/

#include "e1000.h"

void SetLoopBackMode(ADAPTER_STRUCT *Adapter, short LoopbackType);
idiag_pro_stat_t Osdep_LoopBackTest(ADAPTER_STRUCT *Adapter,
				idiag_e1000_diag_loopback_test_param_t *Param, int LbType);

#define RegPatternTest(Register, Mask, WriteVal)			\
		for (Pattern = 0; Pattern < Ntests; Pattern++)		\
		{													\
			E1000_WRITE_REG(Register,						\
				(TestPattern[Pattern] & WriteVal));			\
			ReadValue = E1000_READ_REG(Register);			\
			if (ReadValue != (TestPattern[Pattern] & 		\
						WriteVal & Mask))					\
			{												\
				Param->Reg =								\
				(Adapter->MacType < MAC_LIVENGOOD) ?		\
				offsetof(OLD_REGISTERS, Register) :			\
				offsetof(E1000_REGISTERS, Register);		\
				Param->WriteValue = 						\
				(TestPattern[Pattern] & WriteVal);			\
				Param->ReadValue = ReadValue;				\
				return (IDIAG_PRO_STAT_TEST_FAILED);		\
			}												\
		}

#define RegSetAndCheck(Register, Mask, WriteVal)			\
		E1000_WRITE_REG(Register, WriteVal);				\
		ReadValue = E1000_READ_REG(Register);				\
		if ((WriteVal & Mask) != (ReadValue & Mask))		\
		{													\
			Param->Reg =      	                  			\
			(Adapter->MacType < MAC_LIVENGOOD) ?    		\
			offsetof(OLD_REGISTERS, Register) :           	\
			offsetof(E1000_REGISTERS, Register);         	\
			Param->WriteValue = WriteVal & Mask;			\
			Param->ReadValue = ReadValue & Mask;			\
			return (IDIAG_PRO_STAT_TEST_FAILED);     		\
		}
			
/****************************************************************************
* Name:       DiagRegTest 
*
* Description: This routine Performs diagnostic tests on the hardware
* 		registers 
*
* Author:      IntelCorporation
*
* Born on Date:   1/23/2001 
*
* Arguments:   
*      Adapter		Board dependent data structure 
*      Param		Param structure thru which failed diagnostic test
*      			results are reported.
*
* Returns: 
* 	idiag_pro_stat_t	Status of the Diagnostic Test.
*
* Modification log:
* Date      Who  Description
* --------  ---  -------------------------------------------------------- 
*
****************************************************************************/

idiag_pro_stat_t
DiagRegTest(ADAPTER_STRUCT *Adapter,
	idiag_e1000_diag_reg_test_param_t *Param)
{
	ULONG ReadValue, Pattern;
	ULONG Ntests = 4;
	ULONG TestPattern[4] = {0x5A5A5A5A, 0xA5A5A5A5, 0x00000000, 0xFFFFFFFF};
	ULONG WriteVal, Mask, i;


	/* The status register is Read Only
	 * So a write should fail.
	 * Some bits that get toggled are ignored
	 */
#ifdef DIAG_DEBUG
	DEBUGOUT("Register Test\n");
#endif
	ReadValue = (E1000_READ_REG(Status) & (0xFFFFF833));
	E1000_WRITE_REG(Status, (0xFFFFFFFF));
	if(ReadValue != (E1000_READ_REG(Status) & (0xFFFFF833))) {
			Param->Reg = offsetof(E1000_REGISTERS, Status);
			Param->WriteValue = 0xFFFFFFFF;
			Param->ReadValue = ReadValue;

		return(IDIAG_PRO_STAT_TEST_FAILED);
	}

	/* Register Pattern Test */
	RegPatternTest(Fcal, 0xFFFFFFFF, 0xFFFFFFFF);
	RegPatternTest(Fcah, 0x0000FFFF, 0xFFFFFFFF);
	RegPatternTest(Fct, 0x0000FFFF, 0xFFFFFFFF);
	RegPatternTest(Vet, 0x0000FFFF, 0xFFFFFFFF);

	if (Adapter->MacType >= MAC_LIVENGOOD) {
		RegSetAndCheck(Rctl, 0xFFFFFFFF, 0x07FFB3FF);
	}
	else
	{
		RegSetAndCheck(Rctl, 0xFFFFFFFF, 0x01FFFFFF);
	}

	RegSetAndCheck(Rctl, 0xFFFFFFFF, E1000_RCTL_RST);
		
	if(Adapter->MacType >= MAC_LIVENGOOD)
	{
		for (i = 0; i < E1000_RAR_ENTRIES; i++)
		{
			RegPatternTest(Rar[i].Low, 0xFFFFFFFF, 0xFFFFFFFF);
			RegPatternTest(Rar[i].High, 0x8003FFFF, 0xFFFFFFFF);
		}
	}

	for (i = 0; i < E1000_MC_TBL_SIZE; i++)
	{
		RegPatternTest(Mta[i], 0xFFFFFFFF, 0xFFFFFFFF);
	}

	RegSetAndCheck(Rctl, 0xFFFFFFFF, 0x00000000);

	RegPatternTest(Rdtr0, 0x0000FFFF, 0xFFFFFFFF);

	if (Adapter->MacType >= MAC_LIVENGOOD) {
		RegPatternTest(Rdbal0, 0xFFFFFFF0, 0xFFFFFFFF);
	} else {
		RegPatternTest(Rdbal0, 0xFFFFF000, 0xFFFFFFFF);
	}

	RegPatternTest(Rdbah0, 0xFFFFFFFF, 0xFFFFFFFF);
	RegPatternTest(Rdlen0, 0x000FFF80, 0x000FFFFF);
	RegPatternTest(Rdh0, 0x0000FFFF, 0x0000FFFF);
	RegPatternTest(Rdt0, 0x0000FFFF, 0x0000FFFF);

	RegPatternTest(Fcrth, 0x0000FFF8, 0x0000FFF8);

	RegPatternTest(Fcttv, 0x0000FFFF, 0x0000FFFF);

	if (Adapter->MacType >= MAC_LIVENGOOD) 
	{
		RegPatternTest(Txcw, 0xC000FFFF, 0x0000FFFF);
	}
	else
	{
		RegPatternTest(Txcw, 0x0000FFFF, 0x0000FFFF);
	}

		
	RegSetAndCheck(Tctl, 0xFFFFFFFF, 0x003FFFFB);
	RegSetAndCheck(Tctl, 0xFFFFFFFF, 0x00000000);
		
	RegPatternTest(Tipg, 0x3FFFFFFF, 0x3FFFFFFF);	
	
	if (Adapter->MacType >= MAC_LIVENGOOD) 
	{	
		RegPatternTest(Tdbal, 0xFFFFFFF0, 0xFFFFFFFF);	
	}
	else
	{
		RegPatternTest(Tdbal, 0xFFFFF000, 0xFFFFFFFF);	
	}

	RegPatternTest(Tdbah, 0xFFFFFFFF, 0xFFFFFFFF);	

	RegPatternTest(Tdl, 0x000FFF80, 0x000FFFFF);	
	
	if (Adapter->MacType >= MAC_LIVENGOOD)
		RegPatternTest(Tidv, 0x0000FFFF, 0x0000FFFF);

	if (Adapter->MacType < MAC_LIVENGOOD)
	{
		
		RegPatternTest(Rdtr1, 0x0000FFFF, 0xFFFFFFFF);
		RegPatternTest(Rdbal1, 0xFFFFF000, 0xFFFFFFFF);
		RegPatternTest(Rdbah1, 0xFFFFFFFF, 0xFFFFFFFF);
		RegPatternTest(Rdlen1, 0x000FFF80, 0x000FFFFF);
		RegPatternTest(Rdh1, 0x0000FFFF, 0x0000FFFF);
		RegPatternTest(Rdt1, 0x0000FFFF, 0x0000FFFF);
		RegPatternTest(Tqsal, 0xFFFFFFC0, 0xFFFFFFFF);
		RegPatternTest(Tqsah, 0xFFFFFFFF, 0xFFFFFFFF);
	}

	return (IDIAG_PRO_STAT_OK);


}
/****************************************************************************
* Name:       DiagFifoTest 
*
* Description: This routine Performs diagnostic tests on the Packet buffer  
* 		Memory 
*
* Author:      IntelCorporation
*
* Born on Date:   1/23/2001 
*
* Arguments:   
*      Adapter		Board dependent data structure 
*      Param		Param structure thru which failed diagnostic test
*      			results are reported.
*
* Returns: 
* 	idiag_pro_stat_t	Status of the Diagnostic Test.
*
* Modification log:
* Date      Who  Description
* --------  ---  -------------------------------------------------------- 
*
****************************************************************************/
			
idiag_pro_stat_t
DiagFifoTest(ADAPTER_STRUCT *Adapter,
		idiag_e1000_diag_fifo_test_param_t *Param)
{
	ULONG TestPattern[4] = {0x5A5A5A5A, 0xA5A5A5A5, 0xFFFFFFFF, 0x00000000};
	ULONG TestStatus;
	ULONG Reg;
	ULONG Pattern;
	ULONG ReadValue;

#ifdef DIAG_DEBUG
	DEBUGOUT("FiFo Test\n");
#endif

	for(Pattern = 0; Pattern < 4; Pattern++)
	{
		for(Reg = 0; Reg < 0x4000; Reg++)
		{
			E1000_WRITE_REG(Pbm[Reg], TestPattern[Pattern]);
			ReadValue = E1000_READ_REG(Pbm[Reg]);
			if (ReadValue != TestPattern[Pattern])	
			{
				Param->Offset = Reg;
				Param->WriteValue = TestPattern[Pattern];
				Param->ReadValue = ReadValue;
				return (IDIAG_PRO_STAT_TEST_FAILED);
			}
		}
	}
	return (IDIAG_PRO_STAT_OK);
}

/****************************************************************************
* Name:       DiagEepromTest 
*
* Description: This routine Performs diagnostic tests on the contents of  
* 		on board EEPROM 
*
* Author:      IntelCorporation
*
* Born on Date:   1/23/2001 
*
* Arguments:   
*      Adapter		Board dependent data structure 
*      Param		Param structure thru which failed diagnostic test
*      			results are reported.
*
* Returns: 
* 	idiag_pro_stat_t	Status of the Diagnostic Test.
*
* Modification log:
* Date      Who  Description
* --------  ---  -------------------------------------------------------- 
*
****************************************************************************/

idiag_pro_stat_t
DiagEepromTest(ADAPTER_STRUCT *Adapter,
		idiag_e1000_diag_eeprom_test_param_t *Param)
{
	USHORT Checksum = 0;
	USHORT Iteration;

#ifdef DIAG_DEBUG
	DEBUGOUT("EEPROM Test\n");
#endif

	/* Read and add up the contents of the EEPROM */
	for (Iteration = 0; Iteration < (EEPROM_CHECKSUM_REG + 1); Iteration++)
		     Checksum += ReadEepromWord(Adapter, Iteration);
	
	/* If Checksum is not Correct return error else test passed */
	if(Checksum != (USHORT) EEPROM_SUM) {
		Param->ActualChecksum = Checksum;
		Param->ExpectedChecksum = EEPROM_SUM;

#ifdef DIAG_DEBUG
		DEBUGOUT1("Checksum: %x \n", Checksum);
#endif

		return(IDIAG_PRO_STAT_TEST_FAILED);
	} else {
		return(IDIAG_PRO_STAT_OK);
	}
	
}

/****************************************************************************
* Name:       DiagIntrTest 
*
* Description: This routine Performs diagnostic tests on the Interrupt 
* 		registers  
*
* Author:      IntelCorporation
*
* Born on Date:   1/23/2001 
*
* Arguments:   
*      Adapter		Board dependent data structure 
*      Param		Param structure thru which failed diagnostic test
*      			results are reported.
*
* Returns: 
* 	idiag_pro_stat_t	Status of the Diagnostic Test.
*
* Modification log:
* Date      Who  Description
* --------  ---  -------------------------------------------------------- 
*
****************************************************************************/

idiag_pro_stat_t
DiagIntrTest(ADAPTER_STRUCT *Adapter,
		idiag_e1000_diag_intr_test_param_t *Param)
{
	ULONG IcrContents;	
	ULONG Count;	
	ULONG Mask;	

#ifdef DIAG_DEBUG
	DEBUGOUT("Intr Test\n");
#endif
	
	*Param = IDIAG_E1000_INTR_TEST_OK;
	Adapter->IntrTestInProgress = TRUE;
	Adapter->IntrCount = 0;

	/* Disable all the interrupts */
	E1000_WRITE_REG(Imc, 0xFFFFFFFF);

	DelayInMilliseconds(10);

	/* clear the intr cause register */
	IcrContents = E1000_READ_REG(Icr);

	/* Intr register is clear on read, 
	 * So read the Icr register once more And verify the
	 * Contents are zero. The contents should be zero
	 * becoz all the interrupts are disbled and Icr 
	 * was read earlier before clearing all its contents
	 */
	IcrContents = E1000_READ_REG(Icr);

	if (IcrContents != 0)
	{
		/* Something is wrong, Icr has to be zero. 
		 * Since it is not, we cannot execute
		 * any further interrupt related tests.
		 */ 
		*Param = IDIAG_E1000_INTR_TEST_NOT_EXEC;
		return (IDIAG_PRO_STAT_TEST_FAILED);
	}

	/* Test each interrupt */
	for(Count = 0; Count < 10; Count++)
	{
		/* Disable interrupts */
		E1000_WRITE_REG(Imc, 0xFFFFFFFF);
		DelayInMilliseconds(10);

		/* clear the intr cause register */
		IcrContents = E1000_READ_REG(Icr);
		
		/* Test Interrupts while disabled */
		Mask = 0x0001 << Count;

		/* Disable an interrupt by writing 1 to 
		 * corresponding bit of Imc 
		 */
		E1000_WRITE_REG(Imc, (ULONG) Mask);
		DelayInMilliseconds(10);

		/* Cause an interrupt by writing 1 to 
		 * corresponding bit of Imc 
		 */
		E1000_WRITE_REG(Ics, (ULONG) Mask);
		DelayInMilliseconds(10);
		
		/* Read the Cause register contents */
		IcrContents = E1000_READ_REG(Icr);

		/* If there is an Intr reported over the Bus then
		 * the test failed, becoz the caused intr was disabled
		 * and should not be reported by the device on the bus
		 */ 
		if(Adapter->IntrCount > 0)
		{
			*Param = IDIAG_E1000_INTR_TEST_FAILED_WHILE_DISABLED;
			break;
		}

		/* Test Interrupts While enabled */
		E1000_WRITE_REG(Ims, (ULONG) Mask);
		DelayInMilliseconds(10);
		
		E1000_WRITE_REG(Ics, (ULONG) Mask);
		DelayInMilliseconds(10);

		IcrContents = E1000_READ_REG(Icr);

		/* The Intr was enabled So the device should have put
		 * an intr on the bus.
		 */                         
		if(Adapter->IntrCount == 0) {
			*Param = IDIAG_E1000_INTR_TEST_FAILED_WHILE_ENABLED;
			break;
		}
		/* Reset the Count for next test */
		Adapter->IntrCount = 0;
		
		/* Try to cause any other interrupt which is disabled */
		E1000_WRITE_REG(Imc, (ULONG) ~Mask);
		DelayInMilliseconds(10);

		E1000_WRITE_REG(Ics, (ULONG) ~Mask);
		DelayInMilliseconds(10);

		IcrContents = E1000_READ_REG(Icr);

		/* Intr should not be reported on the bus as the caused
		 * intr's are disabled
		 */
		if(Adapter->IntrCount != 0) {
			*Param = IDIAG_E1000_INTR_TEST_FAILED_MASKED_ENABLED;
			break;
		}
	}

	/* Disable all the interrupts */
	E1000_WRITE_REG(Imc, 0xFFFFFFFF);
	Adapter->IntrTestInProgress = FALSE;

	return ((*Param == IDIAG_E1000_INTR_TEST_OK) ? IDIAG_PRO_STAT_OK:
			IDIAG_PRO_STAT_TEST_FAILED);
}


/****************************************************************************
* Name:       DiagLinkTest 
*
* Description: This routine reports Link Status  
*
* Author:      IntelCorporation
*
* Born on Date:   1/23/2001 
*
* Arguments:   
*      Adapter		Board dependent data structure 
*      Param		Param structure thru which failed diagnostic test
*      			results are reported.
*
* Returns: 
* 	idiag_pro_stat_t	Status of the Diagnostic Test.
*
* Modification log:
* Date      Who  Description
* --------  ---  -------------------------------------------------------- 
*
****************************************************************************/
		
idiag_pro_stat_t
DiagLinkTest(ADAPTER_STRUCT *Adapter,
		idiag_e1000_diag_link_test_param_t *Param)
{
	ULONG TxcwContents;
	ULONG CtrlContents;
	ULONG TestStatus;

#ifdef DIAG_DEBUG
	DEBUGOUT("Link Test\n");
#endif
	
	/* Read the Tx command word */
	TxcwContents = E1000_READ_REG(Txcw);
	
	/* Check if Auto Neg is enabled */
	if(TxcwContents & E1000_TXCW_ANE)
	{
		/* Look for link thru Status register or alternatively
		 * reading LOS in Control register
		 */ 
		TestStatus = (E1000_READ_REG(Status) & E1000_STATUS_LU);
		if(TestStatus)
		{
			*Param = IDIAG_E1000_LINK_TEST_UP;
			return (IDIAG_PRO_STAT_OK);
		}
	} else {
	
		TestStatus = (E1000_READ_REG(Ctrl) & E1000_CTRL_SWDPIN1);
		if(!TestStatus)
		{
			*Param = IDIAG_E1000_LINK_TEST_UP;
			return (IDIAG_PRO_STAT_OK);
		}
	}

	/* If Link was not found and LOS is not set then force link */
	if(TxcwContents & E1000_TXCW_ANE)
	{
		TestStatus = (E1000_READ_REG(Ctrl) & E1000_CTRL_SWDPIN1);
		if(!TestStatus)
		{
			TxcwContents = E1000_READ_REG(Txcw);
			TxcwContents = ~E1000_TXCW_ANE;
			E1000_WRITE_REG(Txcw, TxcwContents);
			
			CtrlContents = E1000_READ_REG(Ctrl);
			CtrlContents |= E1000_CTRL_SLU | E1000_CTRL_FD;

			E1000_WRITE_REG(Ctrl, CtrlContents);

			DelayInMilliseconds(10);

			TestStatus = (E1000_READ_REG(Ctrl) & E1000_CTRL_SWDPIN1);
			if(!TestStatus)
			{
				*Param = IDIAG_E1000_LINK_TEST_UP;
				return (IDIAG_PRO_STAT_OK);
			}
		}
	}

	*Param = IDIAG_E1000_LINK_TEST_DOWN;
	return (IDIAG_PRO_STAT_OK);
	/* return (IDIAG_PRO_STAT_TEST_FAILED); */
}


/****************************************************************************
* Name:       DiagLoopbackTest 
*
* Description: This routine Performs Loopback diagnostic tests. 
*
* Author:      IntelCorporation
*
* Born on Date:   1/23/2001 
*
* Arguments:   
*      Adapter		Board dependent data structure 
*      Param		Param structure thru which failed diagnostic test
*      			results are reported.
*
* Returns: 
* 	idiag_pro_stat_t	Status of the Diagnostic Test.
*
* Modification log:
* Date      Who  Description
* --------  ---  -------------------------------------------------------- 
*
****************************************************************************/

idiag_pro_stat_t
DiagLoopbackTest(ADAPTER_STRUCT *Adapter,
		idiag_e1000_diag_loopback_test_param_t *Param)
{
	ULONG IcrContents;
	ULONG TestStatus;

#ifdef DIAG_DEBUG
	DEBUGOUT(" LOOPBACK TEST \n");
#endif

	/* Disable Interrupts*/	
	E1000_WRITE_REG(Imc, 0xFFFFFFFF);

	/* Clear all interrupts */
	IcrContents = E1000_READ_REG(Icr);
	
	if(Param->mode & IDIAG_E1000_DIAG_MAC_LB) {
		/* Setup MAC loopback mode */
		SetLoopBackMode(Adapter, IDIAG_E1000_DIAG_MAC_LB);

		/* Do the Loopback Test */
		Osdep_LoopBackTest(Adapter, Param, IDIAG_E1000_DIAG_MAC_LB);
	}
	if(Param->mode & IDIAG_E1000_DIAG_TCVR_LB) {
		/* Setup External loopback mode */
		SetLoopBackMode(Adapter, IDIAG_E1000_DIAG_TCVR_LB);

		/* Do the Loopback Test */
		Osdep_LoopBackTest(Adapter, Param, IDIAG_E1000_DIAG_TCVR_LB);
	}
	
	/* Check Params */
	if(Param->mac != IDIAG_E1000_LOOPBACK_TEST_OK &&
	   Param->tcvr != IDIAG_E1000_LOOPBACK_TEST_OK &&
  	   Param->ext != IDIAG_E1000_LOOPBACK_TEST_OK) {
	
		return IDIAG_E1000_LOOPBACK_TEST_FAILED;
	} else {
		return IDIAG_E1000_LOOPBACK_TEST_OK;
	}
}

void SetLoopBackMode(ADAPTER_STRUCT *Adapter, short LoopbackType)
{
	ULONG RctlContents;
	USHORT MiiCtrlReg;

	RctlContents = E1000_READ_REG(Rctl);		

	/* Clear LBM bits */
	RctlContents &= 0xFFFFFF3F;		

	/* Store Bad Packets */
	RctlContents |= E1000_RCTL_SBP;
	RctlContents |= E1000_RCTL_BAM;

	switch(LoopbackType)
	{
		case IDIAG_E1000_DIAG_MAC_LB:
			RctlContents |= E1000_RCTL_LBM_MAC;
			E1000_WRITE_REG(Rctl, RctlContents);
			break;

		case IDIAG_E1000_DIAG_TCVR_LB:
			RctlContents |= E1000_RCTL_LBM_TCVR;
			E1000_WRITE_REG(Rctl, RctlContents);
			if(Adapter->MediaType == MEDIA_TYPE_COPPER) {
					MiiCtrlReg = ReadPhyRegister(Adapter,
								PHY_MII_CTRL_REG, Adapter->PhyAddress);
					MiiCtrlReg |= MII_CR_LOOPBACK;
					WritePhyRegister(Adapter,
									PHY_MII_CTRL_REG, Adapter->PhyAddress,
								   	MiiCtrlReg);
					DelayInMilliseconds(100);
			}
			break;
	}
}

/* OS dependent diagnostic code */

#include "e1000.h"

void
e1000_diag_ioctl(struct net_device *netdev, struct ifreq *ifr)
{
	PADAPTER_STRUCT Adapter = netdev->priv;

	idiag_pro_data_t *DiagData = (idiag_pro_data_t *)ifr->ifr_data;

	if(DiagData->interface_ver != IDIAG_PRO_VERSION) {
		/* incorrect diagnostics interface version */
		DiagData->status = IDIAG_PRO_STAT_NOT_SUPPORTED;
		return;
	}

	if((DiagData->cmd != IDIAG_PRO_IDENTIFY_DRIVER) &&
	   (DiagData->driver_id != IDIAG_E1000_DRIVER)) {
		/* incorrect driver identifier */
		DiagData->status = IDIAG_PRO_STAT_NOT_SUPPORTED;
		return;
	}

	/* Save the Adapter state before starting the test */
	e1000_hibernate_adapter(netdev);
	
	switch (DiagData->cmd) {

	case IDIAG_PRO_IDENTIFY_DRIVER:
		DiagData->driver_id = IDIAG_E1000_DRIVER;
		DiagData->status = IDIAG_PRO_STAT_OK;
		break;	

	case IDIAG_E1000_DIAG_REG_TEST:
		DiagData->status = DiagRegTest(Adapter, (idiag_e1000_diag_reg_test_param_t *)
							           &DiagData->diag_param);
		break;

	case IDIAG_E1000_DIAG_FIFO_TEST:
		DiagData->status = DiagFifoTest(Adapter, (idiag_e1000_diag_fifo_test_param_t *)
						                &DiagData->diag_param);
		break;

	case IDIAG_E1000_DIAG_XSUM_TEST:
		DiagData->status = DiagEepromTest(Adapter, (idiag_e1000_diag_eeprom_test_param_t *)
							              &DiagData->diag_param);
		break;

	case IDIAG_E1000_DIAG_INTR_TEST:
		if(!test_bit(E1000_BOARD_OPEN, &Adapter->flags)) {
			if (request_irq(netdev->irq, &e1000_intr, SA_SHIRQ,
						    "e1000", netdev) != 0) {
				DiagData->status = IDIAG_PRO_STAT_INVALID_STATE;
				break;
			}
		}

		DiagData->status = DiagIntrTest(Adapter, (idiag_e1000_diag_intr_test_param_t *)
										&DiagData->diag_param);

		if(!test_bit(E1000_BOARD_OPEN, &Adapter->flags)) {
			free_irq(netdev->irq, netdev);
		}
		break;

	case IDIAG_E1000_DIAG_LOOPBACK_TEST:
		/* Initialize the adapter */
		if(test_bit(E1000_BOARD_OPEN, &Adapter->flags)) {
			e1000_wakeup_adapter(netdev);
		} else {
			if(e1000_open(netdev) != 0) {
				DiagData->status = IDIAG_PRO_STAT_INVALID_STATE;
				break;
			}
			/* so we know to call e1000_close */
        	clear_bit(E1000_BOARD_OPEN, &Adapter->flags);
		}
#ifdef DIAG_DEBUG
		printk("LBTEST\n");
#endif
		/* Stop the interface from calling Tx entry point */
		netif_stop_queue(netdev);

		DelayInMilliseconds(100);

		DiagData->status = DiagLoopbackTest(Adapter, (idiag_e1000_diag_loopback_test_param_t *)
										    &DiagData->diag_param);
				
#ifdef DIAG_DEBUG
		printk("RES = %x\n", DiagData->status);
#endif
		if(test_bit(E1000_BOARD_OPEN, &Adapter->flags)) {
			e1000_hibernate_adapter(netdev);
		} else {
        	set_bit(E1000_BOARD_OPEN, &Adapter->flags);
			e1000_close(netdev);
		}
		break;

	case IDIAG_E1000_DIAG_LINK_TEST:
		e1000_wakeup_adapter(netdev);

		DelayInMilliseconds(100);

		DiagData->status = DiagLinkTest(Adapter, (idiag_e1000_diag_link_test_param_t *)
						     			&DiagData->diag_param);

		e1000_hibernate_adapter(netdev);
		break;

	default:
		DiagData->status = IDIAG_PRO_STAT_NOT_SUPPORTED;
		break;
	}

	e1000_wakeup_adapter(netdev);

	return;
}

idiag_pro_stat_t
Osdep_LoopBackTest(ADAPTER_STRUCT *Adapter,
				idiag_e1000_diag_loopback_test_param_t *Param, int LbType)
{
	USHORT xmit_stat, rcv_stat;
	ULONG xmit_pkts;
	ULONG rcv_pkts;
	struct sk_buff *skb;

#ifdef DIAG_DEBUG
	printk("Allocating skb\n");
#endif

	skb = alloc_skb(1024, GFP_ATOMIC);

#ifdef DIAG_DEBUG
	printk("len = %d \n", skb->len);
#endif

	skb_put(skb, 1024);

	memset(skb->data, 0xFF, 1024);

	memset(&skb->data[512],0xBA, 511);
 
#ifdef DIAG_DEBUG
	printk("len = %d \n", skb->len);
#endif

	ClearHwStatsCounters(Adapter);

	xmit_stat = e1000_xmit_lbtest_frame(skb, Adapter);
		
#ifdef DIAG_DEBUG
	printk("xmit_stat = %d \n", xmit_stat);
#endif
	
	if (xmit_stat != TRUE) {
		switch(LbType) {
		case IDIAG_E1000_DIAG_MAC_LB:
			Param->mac = IDIAG_E1000_LOOPBACK_TEST_NOT_EXEC;
			break;
		case IDIAG_E1000_DIAG_TCVR_LB:
			Param->tcvr = IDIAG_E1000_LOOPBACK_TEST_NOT_EXEC;
			break;
		case IDIAG_E1000_DIAG_EXT_LB:
			Param->ext = IDIAG_E1000_LOOPBACK_TEST_NOT_EXEC;
			break;
		}
		return (IDIAG_PRO_STAT_TEST_FAILED);
	}

	DelayInMilliseconds(100);

#ifdef DIAG_DEBUG
	xmit_pkts = E1000_READ_REG(Tpt);
	DelayInMilliseconds(100);
	rcv_pkts = E1000_READ_REG(Tpr);
#endif

#ifdef DIAG_DEBUG
	printk("XR: %d, %d\n", xmit_pkts, rcv_pkts);

	rcv_pkts = E1000_READ_REG(Rxerrc);
	printk("XR1: %d\n", rcv_pkts);
#endif

	rcv_stat = e1000_rcv_lbtest_frame(Adapter);
	
	if (rcv_stat != TRUE) {
		switch(LbType) {
		case IDIAG_E1000_DIAG_MAC_LB:
			Param->mac = IDIAG_E1000_LOOPBACK_TEST_FAILED;
			break;
		case IDIAG_E1000_DIAG_TCVR_LB:
			Param->tcvr = IDIAG_E1000_LOOPBACK_TEST_FAILED;
			break;
		case IDIAG_E1000_DIAG_EXT_LB:
			Param->ext = IDIAG_E1000_LOOPBACK_TEST_FAILED;
			break;
		}
		return (IDIAG_PRO_STAT_TEST_FAILED);
	}

	switch(LbType) {
	case IDIAG_E1000_DIAG_MAC_LB:
		Param->mac = IDIAG_E1000_LOOPBACK_TEST_OK;
		break;
	case IDIAG_E1000_DIAG_TCVR_LB:
		Param->tcvr = IDIAG_E1000_LOOPBACK_TEST_OK;
		break;
	case IDIAG_E1000_DIAG_EXT_LB:
		Param->ext = IDIAG_E1000_LOOPBACK_TEST_OK;
		break;
	}
	return (IDIAG_PRO_STAT_OK);
}
